---------------------------------------------------------------------------------------------------
-- Data export script for SimTools V2.4
-- Version 0.09RC3
-- created by @Dirty, find me here: https://www.xsimulator.net/community/members/dirty.27556/
-- Discussions here: https://www.xsimulator.net/community/threads/experimental-dcs-plugin-ready-for-testing.13560/
--
-- Changes:
-- GameVibe & DashBoard support
---------------------------------------------------------------------------------------------------
version = "v0.09RC3"

--Don't change theses values!!!
r2d = 57.295779513082323		--converts Rad into Degrees 
m2k = 1.94384					--converts m/s into Knots

--Change those accordigng to your preference
YouPreferDegreesOverRadians = true
YouPreferKnotsOverMetersPerSecond = true
YouAbsolutelyInsistOnUsingEulerAnglesOnYawEvenThoughThatMakesNoFuckinSense = false
YouAbsolutelyInsistOnUsingEulerAnglesOnPitchEvenThoughThatMakesNoFuckinSense = false
YouAbsolutelyInsistOnUsingEulerAnglesOnRollEvenThoughThatMakesNoFuckinSense = false

--The callbacks to be registered with DCS
local SimTools_Callbacks = {}

function SimTools_Callbacks.onSimulationStart()

	log.write('SimTools', log.INFO, "Starting motion data export. Export script " .. version)
	
	package.path = package.path..";.\\LuaSocket\\?.lua"
	package.cpath = package.cpath..";.\\LuaSocket\\?.dll"
	socket = require("socket")

	IP = "127.0.0.1"
	Port = 41230
	MySocket = socket.udp ( )
	MySocket:settimeout ( 0 )
	MySocket:setpeername ( IP, Port )
end

function SimTools_Callbacks.onSimulationFrame()

	--Accelerations:
	accel = Export.LoGetAccelerationUnits()
		accel_longitudinal	= accel.x
		accel_vertical		= accel.y
		accel_lateral		= accel.z
		
	--Euler Angles:
	pitch, bank, yaw = Export.LoGetADIPitchBankYaw()

	--Angular rates:
	omega = Export.LoGetAngularVelocity()
		omega_longitudinal	= omega.x
		omega_vertical		= omega.y
		omega_lateral		= omega.z



	--Weight On Wheels?
	LeftGear = Export.LoGetAircraftDrawArgumentValue(6)
	NoseGear = Export.LoGetAircraftDrawArgumentValue(1)
	RightGear = Export.LoGetAircraftDrawArgumentValue(4)
	
	--OnGround status is determined by the compression of ANY gear strut
	if (LeftGear > 0 or NoseGear > 0 or RightGear > 0)
		then
			OnGround = 1
		else
			OnGround = 0
	end

	--Groundspeed:
	vv = Export.LoGetVectorVelocity()
	GS = math.sqrt( math.pow(vv.x,2) + math.pow(vv.z,2))
	
	--Air Data
	AOA = Export.LoGetAngleOfAttack()
	IAS = Export.LoGetIndicatedAirSpeed()

	if (YouPreferKnotsOverMetersPerSecond) then
		IAS = IAS * m2k
	end

	
	--Euler angles? Anyone?
	if (YouAbsolutelyInsistOnUsingEulerAnglesOnYawEvenThoughThatMakesNoFuckinSense == true) then
		Yaw_Output = yaw
	else
		Yaw_Output = omega_vertical
	end
		
		
	if (YouAbsolutelyInsistOnUsingEulerAnglesOnPitchEvenThoughThatMakesNoFuckinSense == true) then
		Pitch_Output = pitch
	else
		Pitch_Output = omega_lateral
	end	
	
	
	if (YouAbsolutelyInsistOnUsingEulerAnglesOnRollEvenThoughThatMakesNoFuckinSense == true) then
		Roll_Output = bank
	else
		Roll_Output = omega_longitudinal
	end	
	
	
	--Any unit conversions necessary?
	if (YouPreferDegreesOverRadians) then
		Roll_Output			= Roll_Output			* r2d
		Yaw_Output			= Yaw_Output			* r2d
		Pitch_Output		= Pitch_Output			* r2d
	end
	
	--Change exported values when on ground (optional)
	if (OnGround == 1) then
		accel_longitudinal	= accel_longitudinal	* 1.00
		accel_vertical		= accel_vertical		* 1.00
		accel_lateral		= accel_lateral			* 1.00
		Roll_Output			= Roll_Output			* 1.00
		Yaw_Output			= Yaw_Output			* 1.00
		Pitch_Output		= Pitch_Output			* 1.00
	end	
	
	--==================== CUSTOM EXTENSION FOR TF-51D ===================================
	--Get main panel for accessing cockpit-related arguments
	mainPanel = Export.GetDevice(0)
	
	--Arguments for TF-51D
	engineRPM = mainPanel:get_argument_value(23) -- Range: 0.0 (engine off) to 1.0 (max RPM)
	canopyValue = mainPanel:get_argument_value(162) -- Range: 0.0 (canopy fully closed) to 1.0 (canopy fully opened)
	landingGearGreenLightOn = mainPanel:get_argument_value(80) -- = 1 when gear is extracted (green light is on); = 0 when gear is retracted or during extraction/retraction (green light is off)
	landingGearRedLightOn = mainPanel:get_argument_value(82) -- = 1 when gear is extracting or retracting (red light is on); = 0 when gear is fully extracted or retracted (red light is off); 0 < n < 1 when on ground (red light is off)
	leftCockpitLight = mainPanel:get_argument_value(185)
	--================ END CUSTOM EXTENSION FOR TF-51D ===================================

	--Put data to be exported into these slots. You can put any data you like in there :-)
	--GameEngine:
	local slot_GE_01 = Roll_Output
	local slot_GE_02 = Yaw_Output
	local slot_GE_03 = Pitch_Output
	local slot_GE_04 = accel_longitudinal
	local slot_GE_05 = accel_vertical
	local slot_GE_06 = accel_lateral
	local slot_GE_07 = 0
	local slot_GE_08 = 0
	local slot_GE_09 = 0
	--GameVibe:
	local slot_GV_01 = EngineRPM()
	local slot_GV_02 = 0
	local slot_GV_03 = LandingGearExtractingOrRetracting()
	local slot_GV_04 = WheelSpin()
	local slot_GV_05 = 0
	local slot_GV_06 = 0
	local slot_GV_07 = ElectricalPowerOn()
	local slot_GV_08 = 0
	--GameDash:
	local slot_GD_01 = WindInsideCockpit()
	local slot_GD_02 = Windiness()
	local slot_GD_03 = 0
	local slot_GD_04 = 0
	local slot_GD_05 = 0
	local slot_GD_06 = 0
	local slot_GD_07 = 0
	local slot_GD_08 = 0
	local slot_GD_09 = 0
	local slot_GD_10 = 0
	local slot_GD_11 = 0
	local slot_GD_12 = 0
	local slot_GD_13 = 0
	local slot_GD_14 = 0
	local slot_GD_15 = 0
	local slot_GD_16 = 0
	local slot_GD_17 = 0
	local slot_GD_18 = 0
	local slot_GD_19 = 0
	local slot_GD_20 = 0
	
	--Send data via UDP
	if MySocket then
	--								Slots:      GE01  GE02  GE03  GE04  GE05  GE06  GE07  GE08  GE09  GV01  GV02  GV03  GV04  GV05  GV06  GV07  GV08  GD01  GD02  GD03  GD04  GD05  GD06  GD07  GD08  GD09  GD10  GD11  GD12  GD13  GD14  GD15  GD16  GD17  GD18  GD19  GD20  
		socket.try(MySocket:send(string.format("%.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; %.4f; \n", 
												slot_GE_01,
												slot_GE_02,
												slot_GE_03,
												slot_GE_04,
												slot_GE_05,
												slot_GE_06,
												slot_GE_07,
												slot_GE_08,
												slot_GE_09,
												slot_GV_01,
												slot_GV_02,
												slot_GV_03,
												slot_GV_04,
												slot_GV_05,
												slot_GV_06,
												slot_GV_07,
												slot_GV_08,
												slot_GD_01,
												slot_GD_02,
												slot_GD_03,
												slot_GD_04,
												slot_GD_05,
												slot_GD_06,
												slot_GD_07,
												slot_GD_08,
												slot_GD_09,
												slot_GD_10,
												slot_GD_11,
												slot_GD_12,
												slot_GD_13,
												slot_GD_14,
												slot_GD_15,
												slot_GD_16,
												slot_GD_17,
												slot_GD_18,
												slot_GD_19,
												slot_GD_20
												)))
	end
end

function SimTools_Callbacks.onSimulationStop()
	
	log.write('SimTools', log.INFO, "Motion data export stopped")

	if MySocket then
		MySocket:close()
	end
end

--register these callbacks with DCS to be called at the appropriate moment.
DCS.setUserCallbacks(SimTools_Callbacks)




------------------------------------------------------------------------------------
-----------------------------   HELPER FUNCTIONS   ---------------------------------
------------------------------------------------------------------------------------


function WheelSpin()
	if (OnGround == 1)
		then
			return GS
		else
			return 0
	end	
	
end

function Windiness()
	return IAS		--I'm sure someone will be able to come up with an even more sophisticated formula :-)
end

--==================== CUSTOM EXTENSION FOR TF-51D ===================================
-- The wind increases with increasing engine RPM and when opening the canopy.
function WindInsideCockpit()
	-- The following formula is meant for the TF-51D.
	-- It is adjusted in a way that it results in a bit less than 1.0 when
	-- the canopy is fully opened and the engine's RPM are about maximum.
	return engineRPM * (0.7 + canopyValue * 0.7)
end

-- Returns 1 when gear is extracting or retracting; else returns 0
function LandingGearExtractingOrRetracting()
	if (OnGround == 1) then
		return 0
	else
		return landingGearRedLightOn
	end
end

function EngineRPM()
	return engineRPM
end

function ElectricalPowerOn()
	if (leftCockpitLight > 0) then
		return 1
	else
		return 0
	end
end
--================ END CUSTOM EXTENSION FOR TF-51D ===================================